#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
#pylint: disable-msg=C0103

"""
Config utilities
"""

__revision__ = "$Id: FMWSConfig.py,v 1.5 2009/09/02 16:26:12 valya Exp $"
__version__ = "$Revision: 1.5 $"
__author__ = "Valentin Kuznetsov"

import os
import ConfigParser
from optparse import OptionParser

def fm_configfile():
    """
    Return FMWS configuration file name $FMWSHOME/fmws.cfg
    """
    if  os.environ.has_key('FMWSHOME'):
        fmconfig = os.path.join(os.environ['FMWSHOME'], 'fmws.cfg')
        return fmconfig
    else:
        raise EnvironmentError('FMWSHOME environment is not set up')

def fm_config(iconfig=None):
    """
    FileMover configuration
    """
    if  not iconfig:
        iconfig = {
        'loggerdir': '%s/pool/logs' % os.environ['FMWSHOME'],
        'cmsswdir': '%s/pool/cmssw' % os.environ['FMWSHOME'],
        'verbose': '1',
        'maxtransfer': '3',
        'daytransfer': '10',
        'base_directory': '%s/pool/tmp' % os.environ['FMWSHOME'],
        'max_size_gb': '20',
        'transfer_command': 
            'srmcp -debug=true -srm_protocol_version=2 -retry_num=1 -streams_num=1',
        'priority_0': 'T1_US',
        'priority_1': 'T2',
        'priority_2': 'T1',
        }

    os.makedirs(iconfig['loggerdir'])
    os.makedirs(iconfig['cmsswdir'])
    os.makedirs(iconfig['base_directory'])

    config = ConfigParser.ConfigParser()

    config.add_section('fmws')
    config.set('fmws', 'loggerdir', iconfig['loggerdir'])
    config.set('fmws', 'cmsswdir', iconfig['cmsswdir'])
    config.set('fmws', 'verbose', iconfig['verbose'])
    config.set('fmws', 'maxtransfer', iconfig['maxtransfer'])
    config.set('fmws', 'daytransfer', iconfig['daytransfer'])

    config.add_section('file_manager')
    config.set('file_manager', 'base_directory', iconfig['base_directory'])
    config.set('file_manager', 'max_size_gb', iconfig['max_size_gb'])
    config.add_section('transfer_wrapper')
    config.set('transfer_wrapper', 'transfer_command', iconfig['transfer_command'])
    config.add_section('file_lookup')
    config.set('file_lookup', 'priority_0', iconfig['priority_0'])
    config.set('file_lookup', 'priority_1', iconfig['priority_1'])
    config.set('file_lookup', 'priority_2', iconfig['priority_2'])

    return config

def fm_writeconfig(iconfig=None):
    """
    Write FileMover configuration file
    """
    fmconfig = fm_configfile()
    config   = fm_config(iconfig)
    config.write(open(fmconfig, 'wb'))

class FMWSConfig:
    """
    handle FileMover WebService configuration
    """
    def __init__(self):
        """
        Read and parse content of FMWS.conf configuration file
        """
        fmconfig = fm_configfile()
        self.config = ConfigParser.ConfigParser()
        self.config.read(fmconfig)

    def transferDir(self):
        """
        return transfer dir to be used for FileMover
        """
        return self.config.get('file_manager', 'base_directory', '/tmp/filemover/tmp')
        
    def loggerDir(self):
        """
        return logger dir to be used for FileMover
        """
        return self.config.get('fmws', 'loggerdir', '/tmp/filemover/logs')
        
    def cmsswDir(self):
        """
        return cmssw dir to be used for FileMover
        """
        return self.config.get('fmws', 'cmsswdir', '/tmp/filemover/cmssw')
        
    def verboseLevel(self):
        """
        return verbosity level to be used in FileMover
        """
        return int(self.config.get('fmws', 'verbose', 1))

    def maxTransfer(self):
        """
        return max transfer to be used in FileMover
        """
        return int(self.config.get('fmws', 'maxtransfer', 3))

    def dayTransfer(self):
        """
        return transfers per day to be used in FileMover
        """
        return int(self.config.get('fmws', 'daytransfer', 10))

class MyOptionParser: 
    """
    Simple option parser
    """
    def __init__(self):
        self.parser = OptionParser()
        self.parser.add_option("--config", action="store", type="string", 
                                           default="local", dest="config",
        help="generate fmws.cfg for specific site, e.g. local, CERN, FNAL")

    def getOpt(self):
        """
        Returns parse list of options
        """
        return self.parser.parse_args()

#
# main
#
if __name__ == "__main__":
    optManager  = MyOptionParser()
    (opts,args) = optManager.getOpt()

    iconfig = {
    'loggerdir': '/data/pool/logs',
    'cmsswdir': '/data/pool/cmssw',
    'verbose': '1',
    'maxtransfer': '3',
    'daytransfer': '10',
    'base_directory': '/data/pool',
    'max_size_gb': '20',
    'transfer_command': 
        'srmcp -debug=true -srm_protocol_version=2 -retry_num=1 -streams_num=1',
    'priority_0': 'T1_US',
    'priority_1': 'T2',
    'priority_2': 'T1',
    }

    if  opts.config.lower() == 'cern':
        fm_writeconfig(iconfig)
        print "fmws.cfg for CERN is generated"
    elif  opts.config.lower() == 'fnal':
        iconfig['loggerdir'] = '/storage/local/data2/logs'
        iconfig['cmsswdir'] = '/storage/local/data2/cmssw'
        iconfig['base_directory'] = '/storage/local/data2/pool'
        fm_writeconfig(iconfig)
        print "fmws.cfg for FNAL is generated"
    else:
        fm_writeconfig()
        print "fmws.cfg for localhost is generated"
